/* Copyright (C) 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
   Contributed by Richard Henderson <rth@redhat.com>.

   This file is part of the GNU OpenMP Library (libgomp).

   Libgomp is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   Libgomp is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
   FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.

   Under Section 7 of GPL version 3, you are granted additional
   permissions described in the GCC Runtime Library Exception, version
   3.1, as published by the Free Software Foundation.

   You should have received a copy of the GNU General Public License and
   a copy of the GCC Runtime Library Exception along with this program;
   see the files COPYING3 and COPYING.RUNTIME respectively.  If not, see
   <http://www.gnu.org/licenses/>.  

   MutekH port by Alexandre Becoulet, 2010

*/

#ifndef OMP_H
#define OMP_H 1

#include <stdint.h>

typedef  int_fast32_t  omp_int_t  ;
typedef  uint_fast32_t omp_uint_t ;
typedef  intptr_t      omp_long_t ;
typedef  uintptr_t     omp_ulong_t;

#ifdef CONFIG_OPENMP_USE_POSIX

# include <pthread.h>
# include <semaphore.h>

typedef pthread_mutex_t omp_lock_25_t;
typedef struct { pthread_mutex_t lock; omp_int_t count; } omp_nest_lock_25_t;
typedef sem_t omp_lock_t;
typedef struct { sem_t lock; omp_int_t count; void *owner; } omp_nest_lock_t;

#endif

typedef enum omp_sched_t
{
  omp_sched_static = 1,
  omp_sched_dynamic = 2,
  omp_sched_guided = 3,
  omp_sched_auto = 4
} omp_sched_t;

#ifdef __cplusplus
extern "C" {
# define __GOMP_NOTHROW throw ()
#else
# define __GOMP_NOTHROW __attribute__((__nothrow__))
#endif

void initialize_libgomp (void);

extern void omp_set_num_threads (omp_int_t) __GOMP_NOTHROW;
extern omp_int_t omp_get_num_threads (void) __GOMP_NOTHROW;
extern omp_int_t omp_get_max_threads (void) __GOMP_NOTHROW;
extern omp_int_t omp_get_thread_num (void) __GOMP_NOTHROW;
extern omp_int_t omp_get_num_procs (void) __GOMP_NOTHROW;

extern omp_int_t omp_in_parallel (void) __GOMP_NOTHROW;

extern void omp_set_dynamic (omp_int_t) __GOMP_NOTHROW;
extern omp_int_t omp_get_dynamic (void) __GOMP_NOTHROW;

extern void omp_set_nested (omp_int_t) __GOMP_NOTHROW;
extern omp_int_t omp_get_nested (void) __GOMP_NOTHROW;

extern void omp_init_lock (omp_lock_t *) __GOMP_NOTHROW;
extern void omp_destroy_lock (omp_lock_t *) __GOMP_NOTHROW;
extern void omp_set_lock (omp_lock_t *) __GOMP_NOTHROW;
extern void omp_unset_lock (omp_lock_t *) __GOMP_NOTHROW;
extern omp_int_t omp_test_lock (omp_lock_t *) __GOMP_NOTHROW;

extern void omp_init_nest_lock (omp_nest_lock_t *) __GOMP_NOTHROW;
extern void omp_destroy_nest_lock (omp_nest_lock_t *) __GOMP_NOTHROW;
extern void omp_set_nest_lock (omp_nest_lock_t *) __GOMP_NOTHROW;
extern void omp_unset_nest_lock (omp_nest_lock_t *) __GOMP_NOTHROW;
extern omp_int_t omp_test_nest_lock (omp_nest_lock_t *) __GOMP_NOTHROW;

extern double omp_get_wtime (void) __GOMP_NOTHROW;
extern double omp_get_wtick (void) __GOMP_NOTHROW;

void omp_set_schedule (omp_sched_t, omp_int_t) __GOMP_NOTHROW;
void omp_get_schedule (omp_sched_t *, omp_int_t *) __GOMP_NOTHROW;
omp_int_t omp_get_thread_limit (void) __GOMP_NOTHROW;
void omp_set_max_active_levels (omp_int_t) __GOMP_NOTHROW;
omp_int_t omp_get_max_active_levels (void) __GOMP_NOTHROW;
omp_int_t omp_get_level (void) __GOMP_NOTHROW;
omp_int_t omp_get_ancestor_thread_num (omp_int_t) __GOMP_NOTHROW;
omp_int_t omp_get_team_size (omp_int_t) __GOMP_NOTHROW;
omp_int_t omp_get_active_level (void) __GOMP_NOTHROW;

#ifdef __cplusplus
}
#endif

#endif /* OMP_H */
