/* Copyright (C) 1991, 92, 93, 95, 96, 97, 99 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef NETINET_ICMP_H_
#define NETINET_ICMP_H_

/**
   @file
   @module{Network library}
   @short Standard header for ICMP definitions
 */

#include <hexo/types.h>

/* Types */
#define ICMP_ECHOREPLY		0	//< Echo Reply			
#define ICMP_DEST_UNREACH	3	//< Destination Unreachable	
#define ICMP_SOURCE_QUENCH	4	//< Source Quench		
#define ICMP_REDIRECT		5	//< Redirect (change route)	
#define ICMP_ECHO		8	//< Echo Request			
#define ICMP_TIME_EXCEEDED	11	//< Time Exceeded		
#define ICMP_PARAMETERPROB	12	//< Parameter Problem		
#define ICMP_TIMESTAMP		13	//< Timestamp Request		
#define ICMP_TIMESTAMPREPLY	14	//< Timestamp Reply		
#define ICMP_INFO_REQUEST	15	//< Information Request		
#define ICMP_INFO_REPLY		16	//< Information Reply		
#define ICMP_ADDRESS		17	//< Address Mask Request		
#define ICMP_ADDRESSREPLY	18	//< Address Mask Reply		
#define NR_ICMP_TYPES		18


/* Codes for UNREACH. */
#define ICMP_NET_UNREACH	0	//< Network Unreachable		
#define ICMP_HOST_UNREACH	1	//< Host Unreachable		
#define ICMP_PROT_UNREACH	2	//< Protocol Unreachable		
#define ICMP_PORT_UNREACH	3	//< Port Unreachable		
#define ICMP_FRAG_NEEDED	4	//< Fragmentation Needed/DF set	
#define ICMP_SR_FAILED		5	//< Source Route failed		
#define ICMP_NET_UNKNOWN	6
#define ICMP_HOST_UNKNOWN	7
#define ICMP_HOST_ISOLATED	8
#define ICMP_NET_ANO		9
#define ICMP_HOST_ANO		10
#define ICMP_NET_UNR_TOS	11
#define ICMP_HOST_UNR_TOS	12
#define ICMP_PKT_FILTERED	13	//< Packet filtered 
#define ICMP_PREC_VIOLATION	14	//< Precedence violation 
#define ICMP_PREC_CUTOFF	15	//< Precedence cut off 
#define NR_ICMP_UNREACH		15	//< instead of hardcoding immediate value 

/* Codes for REDIRECT. */
#define ICMP_REDIR_NET		0	//< Redirect Net			
#define ICMP_REDIR_HOST		1	//< Redirect Host		
#define ICMP_REDIR_NETTOS	2	//< Redirect Net for TOS		
#define ICMP_REDIR_HOSTTOS	3	//< Redirect Host for TOS	

/* Codes for TIME_EXCEEDED. */
#define ICMP_EXC_TTL		0	//< TTL count exceeded		
#define ICMP_EXC_FRAGTIME	1	//< Fragment Reass time exceeded	

/** @this is an ICMP header */
struct icmphdr
{
  uint8_t	type;
  uint8_t	code;
  uint16_t	checksum;
  union {
    struct {
      uint16_t	id;
      uint16_t	sequence;
    } echo;
    uint32_t	gateway;
    struct {
      uint16_t	__unused;
      uint16_t	mtu;
    } frag;
  } un;
} __attribute__ ((packed));

#endif

