
name: soclib_crypto
longname: SoCLib Crypto processor

bound: 32
align: 32

%key: 32
  longname: Cipher key
  direction: rw
  address: 0
  count: 8

%iv: 32
  longname: Cipher IV
  direction: rw
  address: 0x100
  count: 8

%dmasrc: 32
  longname: DMA source address
  direction: rw
  address: 0x200

%dmadst: 32
  longname: DMA destination address
  direction: rw
  address: 0x204

%dmalen: 32
  longname: DMA operation length
  direction: rw
  doc: size of the data in bytes.
  address: 0x208

%op: 32
  longname: Processing start
  direction: w
  address: 0x20C
  doc: The processing starts when the operation id is written to this register

  0x0000: AES128_ECB_CRYPT
  0x0001: AES128_ECB_DECRYPT
  0x0002: AES192_ECB_CRYPT
  0x0003: AES192_ECB_DECRYPT
  0x0004: AES256_ECB_CRYPT
  0x0005: AES256_ECB_DECRYPT

  0x0010: AES128_CBC_CRYPT
  0x0011: AES128_CBC_DECRYPT
  0x0012: AES192_CBC_CRYPT
  0x0013: AES192_CBC_DECRYPT
  0x0014: AES256_CBC_CRYPT
  0x0015: AES256_CBC_DECRYPT

%status: 32
  longname: Coprocessor status
  direction: r
  address: 0x210
  
  %%busy: 0-0
    doc: This flag reads as one when the device is processing data.
  
  %%error: 8-31 
    doc: This field indicates the error status.
    0x0000: NONE
    0x0001: WRITE_IO
    0x0002: READ_IO
    0x0003: READ
    0x0004: WRITE
    0x0005: CRYPTO_CFG
    0x0006: REMAINING

%irqen: 32
  longname: Interrupt enable register
  direction: w
  address: 0x214

  %%done: 0-0
    doc: The device will generate an irq when a processing operation terminates

%irqpend: 32
  longname: Interrupt ack register
  direction: r
  address: union
  doc: Reading from this register indicates which interrupts are pending and reset the pending state

  %%done: 0-0
    doc: A processing operation terminated interrupt is pending

