/* -*- c++ -*-
 *
 * SOCLIB_LGPL_HEADER_BEGIN
 * 
 * This file is part of SoCLib, GNU LGPLv2.1.
 * 
 * SoCLib is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; version 2.1 of the License.
 * 
 * SoCLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with SoCLib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 * 
 * SOCLIB_LGPL_HEADER_END
 *
 * Copyright (c) UPMC, Lip6, Asim
 *         Nicolas Pouillon <nipo@ssji.net>, 2007
 */

#include <signal.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "../include/vci_framebuffer.h"
#include "soclib_endian.h"

namespace soclib {
namespace caba {

using namespace soclib;

#define tmpl(t) template<typename vci_param> t VciFrameBuffer<vci_param>

////////////////////////////
tmpl(bool)::on_write(int seg, 
                     typename vci_param::addr_t addr, 
                     typename vci_param::data_t data, 
                     int be)
{
    if (addr>m_fb_controller.surface_size())
        return false;

    int index = addr / vci_param::B;
    uint32_t *tab = m_fb_controller.surface();
	unsigned int cur = tab[index];
    uint32_t mask = vci_param::be2mask(be);

    tab[index] = (cur & ~mask) | (data & mask);

#ifdef SOCLIB_MODULE_DEBUG
	std::cout << "[" << name() << "] WRITE tab["<< index <<"] = " << ( (cur & ~mask) | (data & mask)) << std::endl;
#endif

	m_defered_timeout = 30;
    return true;
}

////////////////////////////
tmpl(bool)::on_read(int seg, 
                    typename vci_param::addr_t addr, 
                    typename vci_param::data_t &data)
{
    if (addr>m_fb_controller.surface_size())
        return false;

	data = m_fb_controller.surface()[addr / vci_param::B];
	return true;
}

////////////////////////
tmpl(void)::transition()
{
	if (!p_resetn) {
		m_vci_fsm.reset();
		return;
	}
	switch ( m_defered_timeout ) {
	case 0:
		break;
	case 1:
		m_fb_controller.update();
	default:
		--m_defered_timeout;
	}
	
	m_vci_fsm.transition();
}

//////////////////////
tmpl(void)::genMoore()
{
	m_vci_fsm.genMoore();
}

///////////////////////////
tmpl(/**/)::VciFrameBuffer(
    sc_module_name name,
    const IntTab &index,
    const MappingTable &mt,
    unsigned long width,
    unsigned long height,
    int subsampling)
	: caba::BaseModule(name),
      m_seglist(mt.getSegmentList(index)),
	  m_vci_fsm(p_vci, m_seglist),
           m_fb_controller((const char *)name, width, height, subsampling),
      p_clk("clk"),
      p_resetn("resetn"),
      p_vci("vci")
{
    std::cout << "  - Building VciFramebuffer : " << name << std::endl;

    assert( (m_seglist.empty() == false) and
    "VCI_FRAMEBUFFER error : no segment allocated");

    std::list<soclib::common::Segment>::iterator seg;
    for ( seg = m_seglist.begin() ; seg != m_seglist.end() ; seg++ )
    {
        std::cout << "    => segment " << seg->name()
                  << " / base = " << std::hex << seg->baseAddress()
                  << " / size = " << seg->size() << std::endl; 
    }

	m_defered_timeout = 0;

	SC_METHOD(transition);
	dont_initialize();
	sensitive << p_clk.pos();
	
	SC_METHOD(genMoore);
	dont_initialize();
	sensitive << p_clk.neg();

	m_vci_fsm.on_read_write(on_read, on_write);

}

tmpl(/**/)::~VciFrameBuffer()
{
}

}}

// Local Variables:
// tab-width: 4
// c-basic-offset: 4
// c-file-offsets:((innamespace . 0)(inline-open . 0))
// indent-tabs-mode: nil
// End:

// vim: filetype=cpp:expandtab:shiftwidth=4:tabstop=4:softtabstop=4

