name: soclib_spi
longname: SoCLib SPI controller

bound:32
align:32

%ctrl: 32
  longname: Control and configuration register

  %%wsize: 0-4
    doc: Bit width of the SPI transfer word minus one.
    default: 7

  %%lsbf: 8-8
    doc: This bit defines the SPI word transmit bit order
    default: 0
    0: MSBF: MSB first
    1: LSBF: LSB first

  %%ckpol: 9-9
    doc: This bit defines the SPI clock polarity
    default: 0
    0: Rising
    1: Falling

  %%ckpha: 10-10
    doc: This bit defines the SPI clock phase
    default: 0
    0: Normal
    1: Inverse

  %%dpol: 11-11
    doc: When set the MISO and MOSI signal are active low.
    default: 0
    0: Normal
    1: Inverted

  %%norx: 12-12
    doc: When set the RX fifo remains empty.
    default: 0
    0: RX
    1: No RX

  %%reset: 13-13
    doc: When set the SPI controller is in reset state. (fifos cleared, tlen = 0)
    default: 0
    0: Normal
    1: Reset

%config: 32
  direction: r
  doc: This register contains the hardware block instantiation parameters

  %%fsize: 0-2
    doc: Size of the RX and TX fifos
    0: 2 words
    1: 4 words
    2: 8 words
    3: 16 words
    4: 32 words
    5: 64 words
    6: 128 words
    7: 256 words

  %%wsize: 8-12
    doc: Maximum bit width of the SPI transfer word minus one.

  %%gpincnt: 13-17
    doc: Number of general purpose input lines minus one

  %%gpoutcnt: 18-22
    doc: Number of general purpose output lines minus one

%clkdiv: 32
  longname: SPI clock divider
  doc: This indicates the SPI clock divider. spi_clk=(clk/(clkdiv+1)/2)
  default: 0

%fifo: 32
  longname: SPI word FIFO register
  doc: Write access to the TX fifo and read access from the RX fifo.
  doc: TX and RX fifo are updated at the same time when a SPI word transfer completes.
  doc: A new transfer is started only if the RX fifo is not full and the TX fifo is not empty.

%gpout: 32
  longname: General purpose outputs
  doc: This register contains the value of up to 32 general purpose output lines.
  default: 0

  %%pin: 1
    count: 32

%gpin: 32
  longname: General purpose inputs
  doc: When read this register return the logic level of up to 32 input lines.
  doc: When written this register set the interrupt mask of input lines.

  %%pin: 1
    count: 32

%gpirq_mode: 32
  doc: This bit defines the irq sense mode of general purpose input lines.
  default: 0

  %%pin: 1
    count: 32
    0: edge: The input bit of the irq pending register changes to one when an edge is detected on a non-masked input.
    1: level: The input bit of the irq pending register is one if at least one the non-masked input is active.

%gpirq_pol: 32
  doc: This bit defines the irq signal polarity of general purpose input lines.
  default: 0

  %%pin: 1
    count: 32
    0: rising: Rising edge or active high
    1: falling: Falling edge or active low

%cstgl: 32
  longname: Chip select toggle
  doc: This bit mask is xored into the gpout register when the tlen register is loaded and when it decrement from 1 to 0.
  default: 0

  %%pin: 1
    count: 32
    0: none: Don't change the output line
    1: toggle: Toggle the output line

%tlen: 32
  longname: Transfer length
  doc: This register is decreased when a SPI word has been transfered.

%status: 32
  longname: Status

  %%txfull: 0-0
    doc: This bit is set when the TX fifo is full

  %%txempty: 1-1
    doc: This bit is set when the TX fifo is empty

  %%txhalf: 2-2
    doc: This bit is set when the TX fifo is half empty

  %%rxfull: 8-8
    doc: This bit is set when the RX fifo is full

  %%rxempty: 9-9
    doc: This bit is set when the RX fifo is empty

  %%rxhalf: 10-10
    doc: This bit is set when the RX fifo is half full

%irqpend: 32
  longname: Pending IRQ mask
  doc: This register is cleared when read. The irq output line of the SPI controller is active when this register is not zero.

  %%txfull: 0-0
    doc: This bit is set when the TX fifo state changes to full.

  %%txempty: 1-1
    doc: This bit is set when the TX fifo state changes to empty.

  %%txhalf: 2-2
    doc: This bit is set when the TX fifo state changes to half empty.

  %%done: 3-3
    doc: This bit is set when the tlen register changes from 1 to 0.

  %%rxfull: 4-4
    doc: This bit is set when the RX fifo state changes to full.

  %%rxempty: 5-5
    doc: This bit is set when the RX fifo state changes to empty.

  %%rxhalf: 6-6
    doc: This bit is set when the RX fifo state changes to half full.

  %%gpirq: 7-7
    doc: This bit is set when the irq condition is detected on an input line.
    doc: This bit will return to zero if the irq mode is level and no more irq condition is detected.

  %%gpirqn: 24-28
    doc: When the gpirq bit is set, this field indicates the index of the lower input line which triggered the irq. This field is zero when no irq is pending.

%irqmask: 32
  longname: IRQ enable mask
  doc: This register specify which irq lines are enabled

  %%txfull: 0-0

  %%txempty: 1-1

  %%txhalf: 2-2

  %%done: 3-3

  %%rxfull: 4-4

  %%rxempty: 5-5

  %%rxhalf: 6-6

  %%gpirq: 7-7

