
name: rt_timer
longname: Real-time high precision timer

bound: 32
align: 32

%sccnt: 32
  longname: Prescaler counter register
  direction: rw
  address: 0
  doc: Prescaler counter value is decremented on rising edge of the input clock and
  doc: reloaded with a configurable value when it reaches zero.

%scrld: 32
  longname: Prescaler reload register
  direction: rw
  address: 4
  doc: This register holds the reload value for the prescaler counter. The max value is 0xffff.

%cfg: 32
  longname: Device configuration register
  direction: r
  address: 8
  doc: This read only register holds timer component information specific to the target platform

  %%dl_cnt: 0-2
    longname: Deadline registers count
  %%irq: 3-7
    longname: Irq vector number
  %%si: 8-8
    longname: Separate interrupts

%ctrl: 32
  longname: Control register
  direction: rw
  address: 12

  %%ce: 0-0
    longname: Chip enabled
    doc: When cleared, this bit prevents the timer from running

  %%iew: 1-1
    longname: Interrupt enable on deadline write
    doc: When set, deadline interrupt is automatically enabled when its value is updated by writting to the associated dln register

  %%ies: 2-2
    longname: Interrupt enable on deadline set
    doc: When set, deadline interrupt is automatically enabled when its value is updated by writting to the associated dln set register

  %%iea: 3-3
    longname: Interrupt enable on deadline add
    doc: When set, deadline interrupt is automatically enabled when its value is updated by writting to the associated dln add register

  %%iec: 4-4
    longname: Interrupt enable on deadline copy
    doc: When set, deadline interrupt is automatically enabled when its value is updated by writting to copy register

%rtcl: 32
  longname: Main counter lower 32 bits value
  direction: r
  address: 16
  doc: This register contains the lower 32 bits of the current value of the timer and latches its higher 32 bits in the rtctmp register.

%rtch: 32
  longname: Main counter higher 32 bits value
  direction: r
  address: 20
  doc: This register contains the higher 32 bits of the current value of the timer and latches its lower 32 bits in the rtctmp register.

%rtctmp: 32
  longname: 32 bits values scratch register
  direction: rw
  address: 24
  doc: This register is used to store lower or higher 32 bits of 64 bits values when accessing main counter or deadlines values.

# %unused: 32
#   address: 28

%pe: 32
  longname: Period enable register
  direction: rw
  address: 32
  doc: If a bit is set in this register, the corresponding deadline register is updated with addition of the associated period register each time the deadline is reached.

%ie: 32
  longname: Interrupts enable register
  direction: rw
  address: 36
  doc: If a bit is set in this register, an interrupt will be generated when the corresponding deadline is reached.

%ip: 32
  longname: Interrupts pending register
  direction: rw
  address: 40
  doc: A bit is set in this register when an interrupt has been generated by the corresponding deadline. Setting a bit to one in this register clears the interrupt pending flag.

%copy: 32
  longname: Deadline copy command register
  direction: w
  address: 44
  doc: When bit N is written to one in this register, the associated deadline register value is overwritten with the value of the N+1 deadline register. The last deadline register can not be overwritten.

%cancel: 32
  longname: Deadline cancel register
  direction: w
  address: 48
  doc: When bit N is written to one in this register, the corresponding irq enable and pending flags are both cleared.

%dln1: 32
  longname: Deadline1 value register
  direction: rw
  address: 128
  doc: This register contains lower 32 bits of the deadline value.
  doc: When read, the higher 32 bits of the deadline value is latched in the rtctmp register.
  doc: When written to, the deadline register higher 32 bits are set to the value of the rtctmp register.

%dln1s: 32
  longname: Deadline1 set register
  direction: w
  address: 132
  doc: When writting to this register, the associated 64 bits deadline register is set to the sum of the timer current value and the written unsigned value.

%dln1a: 32
  longname: Deadline1 add register
  direction: w
  address: 136
  doc: When writting to this register, the written value is sign-extended to 64 bits and added to the associated deadline value.

%dln1p: 32
  longname: Deadline1 period register
  direction: rw
  address: 140
  doc: The value of this register is added to the deadline value when the associated bit in the pe register is set and the deadline is reached.


%dln2: 32
  longname: Deadline2 value register
  direction: rw
  address: 144

%dln2s: 32
  longname: Deadline2 set register
  direction: w
  address: 148

%dln2a: 32
  longname: Deadline2 add register
  direction: w
  address: 152

%dln2p: 32
  longname: Deadline2 period register
  direction: rw
  address: 156


%dln3: 32
  longname: Deadline3 value register
  direction: rw
  address: 160

%dln3s: 32
  longname: Deadline3 set register
  direction: w
  address: 164

%dln3a: 32
  longname: Deadline3 add register
  direction: w
  address: 168

%dln3p: 32
  longname: Deadline3 period register
  direction: rw
  address: 172


%dln4: 32
  longname: Deadline4 value register
  direction: rw
  address: 176

%dln4s: 32
  longname: Deadline4 set register
  direction: w
  address: 180

%dln4a: 32
  longname: Deadline4 add register
  direction: w
  address: 184

%dln4p: 32
  longname: Deadline4 period register
  direction: rw
  address: 188


%dln5: 32
  longname: Deadline5 value register
  direction: rw
  address: 192

%dln5s: 32
  longname: Deadline5 set register
  direction: w
  address: 196

%dln5a: 32
  longname: Deadline5 add register
  direction: w
  address: 200

%dln5p: 32
  longname: Deadline5 period register
  direction: rw
  address: 204


%dln6: 32
  longname: Deadline6 value register
  direction: rw
  address: 208

%dln6s: 32
  longname: Deadline6 set register
  direction: w
  address: 212

%dln6a: 32
  longname: Deadline6 add register
  direction: w
  address: 216

%dln6p: 32
  longname: Deadline6 period register
  direction: rw
  address: 220


%dln7: 32
  longname: Deadline7 value register
  direction: rw
  address: 224

%dln7s: 32
  longname: Deadline7 set register
  direction: w
  address: 228

%dln7a: 32
  longname: Deadline7 add register
  direction: w
  address: 232

%dln7p: 32
  longname: Deadline7 period register
  direction: rw
  address: 236

